@@ -4,7 +4,6 @@ from argparse import ArgumentParser
 
 import astroplan
 import matplotlib
-import numpy
 from astroplan.plots import plot_airmass
 from astropy.coordinates import SkyCoord, get_moon
 import astropy.units as u
@@ -12,12 +11,12 @@ from astroplan import EclipsingSystem, moon_illumination, Constraint
 import pandas as pd
 import numpy as np
 from astroplan import (Observer, FixedTarget, AtNightConstraint, AltitudeConstraint)
-import datetime as dt
-import lightkurve
 import matplotlib.pyplot as plt
 
 from astropy.time import Time
 from lcbuilder.lcbuilder_class import LcBuilder
+from pytz import timezone, utc
+from timezonefinder import TimezoneFinder
 
 from sherlockpipe.observation_report import ObservationReport
 
@@ -79,6 +78,20 @@ def get_twin(ax):
             return other_ax
     return None
 
+
+def get_offset(lat, lng, datetime):
+    """
+    returns a location's time zone offset from UTC in minutes.
+    """
+    tf = TimezoneFinder()
+    tz_target = timezone(tf.certain_timezone_at(lng=lng, lat=lat))
+    if tz_target is None:
+        return None
+    today_target = tz_target.localize(datetime)
+    today_utc = utc.localize(datetime)
+    return (today_utc - today_target).total_seconds() / 3600
+
+
 def create_observation_observables(object_id, object_dir, name, epoch, epoch_low_err, epoch_up_err, period,
                                    period_low_err, period_up_err, duration,
                                    observatories_file, timezone, latitude, longitude, altitude,
@@ -126,12 +139,10 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
     obs_time = Time.now()
     system = EclipsingSystem(primary_eclipse_time=primary_eclipse_time, orbital_period=u.Quantity(period, unit="d"),
                              duration=u.Quantity(duration, unit="h"), name=name)
-    observables_df = pd.DataFrame(columns=['observatory', 'ingress', 'egress', 'midtime', 'ingress_local',
-                                           'egress_local', 'midtime_local',
+    observables_df = pd.DataFrame(columns=['observatory', 'timezone', 'ingress', 'egress', 'midtime',
                                            "midtime_up_err_h",
                                            "midtime_low_err_h", 'twilight_evening',
-                                           'twilight_morning', 'twilight_evening_local',
-                                           'twilight_morning_local', 'observable', 'moon_phase', 'moon_dist'])
+                                           'twilight_morning', 'observable', 'moon_phase', 'moon_dist'])
     plan_dir = object_dir + "/plan"
     images_dir = plan_dir + "/images"
     if os.path.exists(plan_dir):
@@ -141,13 +152,11 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
         shutil.rmtree(images_dir, ignore_errors=True)
     os.mkdir(images_dir)
     for index, observatory_row in observatories_df.iterrows():
-        observer_timezone = dt.timezone(dt.timedelta(hours=observatory_row["tz"]))
         observer_site = Observer(latitude=observatory_row["lat"], longitude=observatory_row["lon"],
-                                 elevation=u.Quantity(observatory_row["alt"], unit="m"),
-                                 timezone=observer_timezone)
+                                 elevation=u.Quantity(observatory_row["alt"], unit="m"))
         midtransit_times = system.next_primary_eclipse_time(obs_time, n_eclipses=n_transits)
         ingress_egress_times = system.next_primary_ingress_egress_time(obs_time, n_eclipses=n_transits)
-        constraints = [AtNightConstraint.twilight_civil(), AltitudeConstraint(min=min_altitude * u.deg),
+        constraints = [AtNightConstraint.twilight_nautical(), AltitudeConstraint(min=min_altitude * u.deg),
                        MoonIlluminationSeparationConstraint(min_dist=moon_min_dist * u.deg,
                                                             max_dist=moon_max_dist * u.deg)]
         ingress_observables = astroplan.is_event_observable(constraints, observer_site, target, times=ingress_egress_times[:, 0])[0]
@@ -184,17 +193,18 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
             transits_since_epoch = round((midtransit_time - primary_eclipse_time).jd / period)
             midtransit_time_low_err = np.round((((transits_since_epoch * period_low_err) ** 2 + epoch_low_err ** 2) ** (1 / 2)) * 24, 2)
             midtransit_time_up_err = np.round((((transits_since_epoch * period_up_err) ** 2 + epoch_up_err ** 2) ** (1 / 2)) * 24, 2)
-            observables_df = observables_df.append({"observatory": observatory_row["name"], "ingress": ingress.isot,
+            if observatory_row["tz"] is not None and not np.isnan(observatory_row["tz"]):
+                observer_timezone = observatory_row["tz"]
+            else:
+                observer_timezone = get_offset(observatory_row["lat"], observatory_row["lon"],
+                                               midtransit_time.datetime)
+            observables_df = observables_df.append({"observatory": observatory_row["name"],
+                                                    "timezone": observer_timezone, "ingress": ingress.isot,
                                    "egress": egress.isot, "midtime": midtransit_time.isot,
-                                   "ingress_local": ingress.to_datetime(timezone=observer_timezone),
-                                   "egress_local": egress.to_datetime(timezone=observer_timezone),
-                                   "midtime_local": midtransit_time.to_datetime(timezone=observer_timezone),
                                    "midtime_up_err_h": midtransit_time_up_err,
                                    "midtime_low_err_h": midtransit_time_low_err,
                                    "twilight_evening": twilight_evening.isot,
                                    "twilight_morning": twilight_morning.isot,
-                                   "twilight_evening_local": twilight_evening.to_datetime(timezone=observer_timezone),
-                                   "twilight_morning_local": twilight_morning.to_datetime(timezone=observer_timezone),
                                    "observable": observable, "moon_phase": moon_phase, "moon_dist": moon_dist},
                                                    ignore_index=True)
             delta_t = (egress - ingress) * 3
@@ -202,7 +212,7 @@ def create_observation_observables(object_id, object_dir, name, epoch, epoch_low
             end_plot_time = egress + duration * u.hour
             plot_time = init_plot_time + delta_t * np.linspace(0, 1, 50)
             plt.tick_params(labelsize=6)
-            airmass_ax = plot_airmass(target, observer_site, plot_time, brightness_shading=True, altitude_yaxis=True)
+            airmass_ax = plot_airmass(target, observer_site, plot_time, brightness_shading=False, altitude_yaxis=True)
             airmass_ax.axvspan(twilight_morning.plot_date, end_plot_time.plot_date, color='white')
             airmass_ax.axvspan(init_plot_time.plot_date, twilight_evening.plot_date, color='white')
             airmass_ax.axvspan(twilight_evening.plot_date, twilight_morning.plot_date, color='gray')
@@ -252,7 +262,6 @@ if __name__ == '__main__':
     ap.add_argument('--object_dir',
                     help="If the object directory is not your current one you need to provide the ABSOLUTE path",
                     required=False)
-    ap.add_argument('--candidate', help="Candidate number from promising list", required=True)
     ap.add_argument("--observatories", help="Csv file containing the observatories coordinates", required=False)
     ap.add_argument('--lat', help="Observer latitude", required=False)
     ap.add_argument('--lon', help="Observer longitude", required=False)
@@ -275,39 +284,42 @@ if __name__ == '__main__':
     ns_file = object_dir + "/results/ns_table.csv"
     if not os.path.exists(ns_derived_file) or not os.path.exists(ns_file):
         raise ValueError("Bayesian fit posteriors files {" + ns_file + ", " + ns_derived_file + "} not found")
-    fit_derived_results = pd.read_csv(object_dir + "/results/ns_derived_table.csv")
-    fit_results = pd.read_csv(object_dir + "/results/ns_table.csv")
-    period_row = fit_results[fit_results["#name"].str.contains("_period")]
-    period = period_row["median"].item()
-    period_low_err = float(period_row["lower_error"].item())
-    period_up_err = float(period_row["upper_error"].item())
-    epoch_row = fit_results[fit_results["#name"].str.contains("_epoch")]
-    epoch = epoch_row["median"].item()
-    epoch_low_err = float(epoch_row["lower_error"].item())
-    epoch_up_err = float(epoch_row["upper_error"].item())
-    duration_row = fit_derived_results[fit_derived_results["#property"].str.contains("Total transit duration")]
-    duration = duration_row["value"].item()
-    duration_low_err = float(duration_row["lower_error"].item())
-    duration_up_err = float(duration_row["upper_error"].item())
-    depth_row = fit_derived_results[fit_derived_results["#property"].str.contains("depth \(dil.\)")]
-    depth = float(depth_row["value"].item()) * 1000
-    depth_low_err = float(depth_row["lower_error"].item()) * 1000
-    depth_up_err = float(depth_row["upper_error"].item()) * 1000
     star_df = pd.read_csv(object_dir + "/params_star.csv")
     object_id = star_df.iloc[0]["obj_id"]
-    name = object_id + "_SOI_" + str(args.candidate)
     ra = star_df.iloc[0]["ra"]
     dec = star_df.iloc[0]["dec"]
     coords = str(ra) + " " + str(dec)
-    observatories_df, observables_df, plan_dir, images_dir = create_observation_observables(object_id, object_dir, name, epoch,
-                                                        epoch_low_err, epoch_up_err, period, period_low_err,
-                                                        period_up_err, duration, args.observatories, args.tz, args.lat,
-                                                        args.lon, args.alt, args.max_days, args.min_altitude,
-                                                        args.moon_min_dist, args.moon_max_dist, args.transit_fraction)
-    report = ObservationReport(observatories_df, observables_df, object_id, plan_dir,
-                               epoch, epoch_low_err, epoch_up_err, period, period_low_err, period_up_err, duration,
-                               duration_low_err, duration_up_err, depth, depth_low_err, depth_up_err,
-                               args.transit_fraction, args.moon_min_dist, args.moon_max_dist, args.min_altitude,
-                               args.max_days)
-    report.create_report()
-    shutil.rmtree(images_dir, ignore_errors=True)
+    fit_derived_results = pd.read_csv(object_dir + "/results/ns_derived_table.csv")
+    fit_results = pd.read_csv(object_dir + "/results/ns_table.csv")
+    candidates_count = len(fit_results[fit_results["#name"].str.contains("_period")])
+    for i in np.arange(0, candidates_count):
+        period_row = fit_results[fit_results["#name"].str.contains("_period")].iloc[i]
+        period = period_row["median"]
+        period_low_err = float(period_row["lower_error"])
+        period_up_err = float(period_row["upper_error"])
+        name = object_id + "_" + period_row["#name"].replace("_period", "")
+        epoch_row = fit_results[fit_results["#name"].str.contains("_epoch")].iloc[i]
+        epoch = epoch_row["median"].item()
+        epoch_low_err = float(epoch_row["lower_error"])
+        epoch_up_err = float(epoch_row["upper_error"])
+        duration_row = fit_derived_results[fit_derived_results["#property"].str.contains("Total transit duration")].iloc[i]
+        duration = duration_row["value"].item()
+        duration_low_err = float(duration_row["lower_error"])
+        duration_up_err = float(duration_row["upper_error"])
+        depth_row = fit_derived_results[fit_derived_results["#property"].str.contains("depth \(dil.\)")].iloc[i]
+        depth = depth_row["value"] * 1000
+        depth_low_err = depth_row["lower_error"] * 1000
+        depth_up_err = depth_row["upper_error"] * 1000
+        observatories_df, observables_df, plan_dir, images_dir = create_observation_observables(object_id, object_dir, name, epoch,
+                                                            epoch_low_err, epoch_up_err, period, period_low_err,
+                                                            period_up_err, duration, args.observatories, args.tz, args.lat,
+                                                            args.lon, args.alt, args.max_days, args.min_altitude,
+                                                            args.moon_min_dist, args.moon_max_dist, args.transit_fraction)
+        report = ObservationReport(observatories_df, observables_df, object_id, name, plan_dir, ra, dec,
+                                   epoch, epoch_low_err, epoch_up_err, period, period_low_err, period_up_err, duration,
+                                   duration_low_err, duration_up_err, depth, depth_low_err, depth_up_err,
+                                   args.transit_fraction, args.moon_min_dist, args.moon_max_dist, args.min_altitude,
+                                   args.max_days, star_df.iloc[0]["v"], star_df.iloc[0]["j"], star_df.iloc[0]["h"],
+                                   star_df.iloc[0]["k"])
+        report.create_report()
+        shutil.rmtree(images_dir, ignore_errors=True)
